@@ -122,7 +122,7 @@ readRandomInt32Data(UA_Server *server,
     }
     return UA_STATUSCODE_GOOD;
 }
-    
+
 static UA_StatusCode
 readRandomInt64Data(UA_Server *server,
              const UA_NodeId *sessionId, void *sessionContext,
@@ -143,7 +143,7 @@ readRandomInt64Data(UA_Server *server,
     }
     return UA_STATUSCODE_GOOD;
 }
-    
+
 static UA_StatusCode
 readRandomUInt16Data(UA_Server *server,
              const UA_NodeId *sessionId, void *sessionContext,
@@ -185,7 +185,7 @@ readRandomUInt32Data(UA_Server *server,
     }
     return UA_STATUSCODE_GOOD;
 }
-    
+
 static UA_StatusCode
 readRandomUInt64Data(UA_Server *server,
              const UA_NodeId *sessionId, void *sessionContext,
@@ -219,7 +219,7 @@ readRandomStringData (UA_Server *server,
         return UA_STATUSCODE_GOOD;
     }
     char randomName[12];
-    UA_snprintf(randomName, 12, "Random%d", UA_UInt32_random());
+    UA_snprintf(randomName, 12, "Random%u", UA_UInt32_random());
     UA_String toggle = UA_STRING(randomName);
     UA_Variant_setScalarCopy(&value->value, &toggle, &UA_TYPES[UA_TYPES_STRING]);
     value->hasValue = true;
@@ -283,7 +283,7 @@ readByteString (UA_Server *server,
         return UA_STATUSCODE_GOOD;
     }
     char randomName[8];
-    UA_snprintf(randomName, 8, "%d%d", UA_UInt32_random(), UA_UInt32_random());
+    UA_snprintf(randomName, 8, "%u%u", UA_UInt32_random(), UA_UInt32_random());
     UA_ByteString randomByte = UA_BYTESTRING(randomName);
     UA_Variant_setScalarCopy(&value->value, &randomByte, &UA_TYPES[UA_TYPES_BYTESTRING]);
     value->hasValue = true;
@@ -293,7 +293,28 @@ readByteString (UA_Server *server,
     }
    return UA_STATUSCODE_GOOD;
 }
-    
+
+#ifdef UA_ENABLE_SUBSCRIPTIONS_EVENTS
+static UA_NodeId eventId;
+static void
+writeEventTrigger(UA_Server *server, const UA_NodeId *sessionId,
+                  void *sessionContext, const UA_NodeId *nodeId,
+                  void *nodeContext, const UA_NumericRange *range,
+                  const UA_DataValue *data) {
+    UA_Server_triggerEvent(server, eventId,
+                           UA_NODEID_NUMERIC(0, UA_NS0ID_SERVER),
+                           NULL, false);
+}
+
+static void
+cyclicEventTriger(UA_Server *server, void *data) {
+    (void)data;
+    UA_Server_triggerEvent(server, eventId,
+                           UA_NODEID_NUMERIC(0, UA_NS0ID_SERVER),
+                           NULL, false);
+}
+#endif
+
 /* Method Node Example */
 #ifdef UA_ENABLE_METHODCALLS
 
@@ -498,7 +519,7 @@ setInformationModel(UA_Server *server) {
 
         UA_VariableAttributes attr = UA_VariableAttributes_default;
         attr.dataType = UA_TYPES[type].typeId;
-#ifndef UA_ENABLE_TYPENAMES
+#ifndef UA_ENABLE_TYPEDESCRIPTION
         char name[15];
         UA_snprintf(name, 15, "%02d", type);
         attr.displayName = UA_LOCALIZEDTEXT("en-US", name);
@@ -547,7 +568,7 @@ setInformationModel(UA_Server *server) {
                                   UA_NODEID_NUMERIC(0, UA_NS0ID_ORGANIZES), qualifiedName,
                                   baseDataVariableType, attr, NULL, NULL);
         UA_Variant_clear(&attr.value);
-#ifdef UA_ENABLE_TYPENAMES
+#ifdef UA_ENABLE_TYPEDESCRIPTION
         UA_LocalizedText_clear(&attr.displayName);
         UA_QualifiedName_clear(&qualifiedName);
 #endif
@@ -586,7 +607,7 @@ setInformationModel(UA_Server *server) {
     id = DEPTHID; // running id in namespace 0 - Start with Matrix NODE
     for(UA_UInt32 i = 1; i <= 20; i++) {
         char name[15];
-        UA_snprintf(name, 15, "depth%i", i);
+        UA_snprintf(name, 15, "depth%u", i);
         object_attr.description = UA_LOCALIZEDTEXT("en-US", name);
         object_attr.displayName = UA_LOCALIZEDTEXT("en-US", name);
         UA_Server_addObjectNode(server, UA_NODEID_NUMERIC(1, id + i),
@@ -605,70 +626,60 @@ setInformationModel(UA_Server *server) {
             continue;
 
         UA_DataSource scaleTestDataSource;
+        scaleTestDataSource.read = NULL;
+        scaleTestDataSource.write = NULL;
         UA_VariableAttributes attr = UA_VariableAttributes_default;
         attr.dataType = UA_TYPES[type].typeId;
         attr.accessLevel = UA_ACCESSLEVELMASK_READ | UA_ACCESSLEVELMASK_WRITE;
         attr.writeMask = UA_WRITEMASK_DISPLAYNAME | UA_WRITEMASK_DESCRIPTION;
         attr.userWriteMask = UA_WRITEMASK_DISPLAYNAME | UA_WRITEMASK_DESCRIPTION;
         attr.valueRank = UA_VALUERANK_SCALAR;
-        switch(UA_TYPES[type].typeIndex) {
-            case UA_TYPES_BOOLEAN: {
+        switch(UA_TYPES[type].typeKind) {
+            case UA_DATATYPEKIND_BOOLEAN: {
                 scaleTestDataSource.read = readRandomBoolData;
-                scaleTestDataSource.write = NULL;
                 break;
             }
-            case UA_TYPES_INT16: {
+            case UA_DATATYPEKIND_INT16: {
                 scaleTestDataSource.read = readRandomInt16Data;
-                scaleTestDataSource.write = NULL;
                 break;
             }
-            case UA_TYPES_UINT16: {
+            case UA_DATATYPEKIND_UINT16: {
                 scaleTestDataSource.read = readRandomUInt16Data;
-                scaleTestDataSource.write = NULL;
                 break;
             }
-            case UA_TYPES_INT32: {
+            case UA_DATATYPEKIND_INT32: {
                 scaleTestDataSource.read = readRandomInt32Data;
-                scaleTestDataSource.write = NULL;
                 break;
             }
-            case UA_TYPES_UINT32: {
+            case UA_DATATYPEKIND_UINT32: {
                 scaleTestDataSource.read = readRandomUInt32Data;
-                scaleTestDataSource.write = NULL;
                 break;
             }
-            case UA_TYPES_INT64: {
+            case UA_DATATYPEKIND_INT64: {
                 scaleTestDataSource.read = readRandomInt64Data;
-                scaleTestDataSource.write = NULL;
                 break;
             }
-            case UA_TYPES_UINT64: {
+            case UA_DATATYPEKIND_UINT64: {
                 scaleTestDataSource.read = readRandomUInt64Data;
-                scaleTestDataSource.write = NULL;
                 break;
             }
-            case UA_TYPES_STRING: {
+            case UA_DATATYPEKIND_STRING: {
                 scaleTestDataSource.read = readRandomStringData;
-                scaleTestDataSource.write = NULL;
                 break;
             }
-            case UA_TYPES_FLOAT: {
+            case UA_DATATYPEKIND_FLOAT: {
                 scaleTestDataSource.read = readRandomFloatData;
-                scaleTestDataSource.write = NULL;
                 break;
             }
-            case UA_TYPES_DOUBLE: {
+            case UA_DATATYPEKIND_DOUBLE: {
                 scaleTestDataSource.read = readRandomDoubleData;
-                scaleTestDataSource.write = NULL;
                 break;
             }
-            case UA_TYPES_DATETIME:
+            case UA_DATATYPEKIND_DATETIME:
                 scaleTestDataSource.read = readTimeData;
-                scaleTestDataSource.write = NULL;
                 break;
-            case UA_TYPES_BYTESTRING:
+            case UA_DATATYPEKIND_BYTESTRING:
                 scaleTestDataSource.read = readByteString;
-                scaleTestDataSource.write = NULL;
                 break;
             default:
                 break;
@@ -676,7 +687,7 @@ setInformationModel(UA_Server *server) {
 
         for(size_t j = 0; j < 100; j++) {
             char name[32];
-#ifndef UA_ENABLE_TYPENAMES
+#ifndef UA_ENABLE_TYPEDESCRIPTION
             UA_snprintf(name, 20, "%02d - %i", type, scale_i);
 #else
             UA_snprintf(name, 20, "%s - %i", UA_TYPES[type].typeName, scale_i);
@@ -772,6 +783,51 @@ setInformationModel(UA_Server *server) {
                             &outargMethod, /* callback of the method node */
                             1, &inputArguments, 1, &outputArguments, NULL, NULL);
 #endif
+
+#ifdef UA_ENABLE_SUBSCRIPTIONS_EVENTS
+    /* Create the reusable event instance */
+    UA_Server_createEvent(server, UA_NODEID_NUMERIC(0, UA_NS0ID_BASEEVENTTYPE), &eventId);
+    UA_UInt16 eventSeverity = 500;
+    UA_Server_writeObjectProperty_scalar(server, eventId,
+                                         UA_QUALIFIEDNAME(0, "Severity"),
+                                         &eventSeverity, &UA_TYPES[UA_TYPES_UINT16]);
+
+    /* Trigger the event from two variables */
+    UA_ValueCallback eventTriggerValueBackend;
+    eventTriggerValueBackend.onRead = NULL;
+    eventTriggerValueBackend.onWrite = writeEventTrigger;
+
+    UA_VariableAttributes_init(&myVar);
+    myVar.description = UA_LOCALIZEDTEXT("en-US", "event trigger 1");
+    myVar.displayName = UA_LOCALIZEDTEXT("en-US", "event trigger 1");
+    myVar.accessLevel = UA_ACCESSLEVELMASK_READ | UA_ACCESSLEVELMASK_WRITE;
+    myVar.dataType = UA_TYPES[UA_TYPES_INT32].typeId;
+    myVar.valueRank = UA_VALUERANK_SCALAR;
+    myInteger = 0;
+    UA_Variant_setScalar(&myVar.value, &myInteger, &UA_TYPES[UA_TYPES_INT32]);
+    parentNodeId = UA_NODEID_NUMERIC(0, UA_NS0ID_OBJECTSFOLDER);
+    parentReferenceNodeId = UA_NODEID_NUMERIC(0, UA_NS0ID_ORGANIZES);
+    UA_Server_addVariableNode(server, UA_NODEID_STRING(1, "event-trigger-1"),
+                              parentNodeId, parentReferenceNodeId,
+                              UA_QUALIFIEDNAME(1, "event trigger 1"),
+                              baseDataVariableType, myVar, NULL, NULL);
+    UA_Server_setVariableNode_valueCallback(server,
+                                            UA_NODEID_STRING(1, "event-trigger-1"),
+                                            eventTriggerValueBackend);
+
+    myVar.description = UA_LOCALIZEDTEXT("en-US", "event trigger 2");
+    myVar.displayName = UA_LOCALIZEDTEXT("en-US", "event trigger 2");
+    UA_Server_addVariableNode(server, UA_NODEID_STRING(1, "event-trigger-2"),
+                              parentNodeId, parentReferenceNodeId,
+                              UA_QUALIFIEDNAME(1, "event trigger 2"),
+                              baseDataVariableType, myVar, NULL, NULL);
+    UA_Server_setVariableNode_valueCallback(server,
+                                            UA_NODEID_STRING(1, "event-trigger-2"),
+                                            eventTriggerValueBackend);
+
+    /* Auto-trigger the event every 500 ms */
+    UA_Server_addRepeatedCallback(server, cyclicEventTriger, NULL, 500.0, NULL);
+#endif
 }
 
 static void
@@ -948,11 +1004,11 @@ usage(void) {
 #endif
                    "\t[--enableUnencrypted]\n"
                    "\t[--enableOutdatedSecurityPolicy]\n"
-                   "\t[--enableTimestampCheck]\n"
                    "\t[--disableBasic128]\n"
                    "\t[--disableBasic256]\n"
                    "\t[--disableBasic256Sha256]\n"
 #endif
+                   "\t[--enableTimestampCheck]\n"
                    "\t[--enableAnonymous]\n");
 }
 
@@ -1000,7 +1056,6 @@ int main(int argc, char **argv) {
     char filetype = ' '; /* t==trustlist, l == issuerList, r==revocationlist */
     UA_Boolean enableUnencr = false;
     UA_Boolean enableSec = false;
-    UA_Boolean enableTime = false;
     UA_Boolean disableBasic128 = false;
     UA_Boolean disableBasic256 = false;
     UA_Boolean disableBasic256Sha256 = false;
@@ -1016,11 +1071,12 @@ int main(int argc, char **argv) {
     const char *trustlistFolder = NULL;
     const char *issuerlistFolder = NULL;
     const char *revocationlistFolder = NULL;
-#endif
+#endif /* __linux__ */
 
-#endif
+#endif /* UA_ENABLE_ENCRYPTION */
 
     UA_Boolean enableAnon = false;
+    UA_Boolean enableTime = false;
 
     /* Loop over the remaining arguments */
     for(; pos < (size_t)argc; pos++) {
@@ -1030,6 +1086,11 @@ int main(int argc, char **argv) {
             continue;
         }
 
+        if(strcmp(argv[pos], "--enableTimestampCheck") == 0) {
+            enableTime = true;
+            continue;
+        }
+
 #ifdef UA_ENABLE_ENCRYPTION
         if(strcmp(argv[pos], "--enableUnencrypted") == 0) {
             enableUnencr = true;
@@ -1041,11 +1102,6 @@ int main(int argc, char **argv) {
             continue;
         }
 
-        if(strcmp(argv[pos], "--enableTimestampCheck") == 0) {
-            enableTime = true;
-            continue;
-        }
-
         if(strcmp(argv[pos], "--disableBasic128") == 0) {
             disableBasic128 = true;
             continue;
@@ -1059,7 +1115,22 @@ int main(int argc, char **argv) {
         if(strcmp(argv[pos], "--disableBasic256Sha256") == 0) {
             disableBasic256Sha256 = true;
             continue;
-        }        
+        }
+
+        if(strcmp(argv[pos], "--disableBasic128") == 0) {
+            disableBasic128 = true;
+            continue;
+        }
+
+        if(strcmp(argv[pos], "--disableBasic256") == 0) {
+            disableBasic256 = true;
+            continue;
+        }
+
+        if(strcmp(argv[pos], "--disableBasic256Sha256") == 0) {
+            disableBasic256Sha256 = true;
+            continue;
+        }
 
 #ifndef __linux__
         if(strcmp(argv[pos], "--trustlist") == 0) {
@@ -1124,7 +1195,7 @@ int main(int argc, char **argv) {
             revocationListSize++;
             continue;
         }
-#else
+#else /* __linux__ */
         if(strcmp(argv[pos], "--trustlistFolder") == 0) {
             filetype = 't';
             continue;
@@ -1154,30 +1225,40 @@ int main(int argc, char **argv) {
             revocationlistFolder = argv[pos];
             continue;
         }
-#endif
+#endif /* __linux__ */
 
-#endif
+#endif /* UA_ENABLE_ENCRYPTION */
 
         usage();
         return EXIT_FAILURE;
     }
 
+    UA_Server *server = NULL;
+
 #ifdef UA_ENABLE_ENCRYPTION
 #ifndef __linux__
-    UA_ServerConfig_setDefaultWithSecurityPolicies(&config, 4840,
-                                                   &certificate, &privateKey,
-                                                   trustList, trustListSize,
-                                                   issuerList, issuerListSize,
-                                                   revocationList, revocationListSize);
-#else
-    UA_ServerConfig_setDefaultWithSecurityPolicies(&config, 4840,
-                                                   &certificate, &privateKey,
-                                                   NULL, 0, NULL, 0, NULL, 0);
-    config.certificateVerification.deleteMembers(&config.certificateVerification);
-    UA_CertificateVerification_CertFolders(&config.certificateVerification,
-                                           trustlistFolder, issuerlistFolder,
-                                           revocationlistFolder);
-#endif
+    UA_StatusCode res =
+        UA_ServerConfig_setDefaultWithSecurityPolicies(&config, 4840,
+                                                       &certificate, &privateKey,
+                                                       trustList, trustListSize,
+                                                       issuerList, issuerListSize,
+                                                       revocationList, revocationListSize);
+    if(res != UA_STATUSCODE_GOOD)
+        goto cleanup;
+#else /* On Linux we can monitor the certs folder and reload when changes are made */
+    UA_StatusCode res =
+        UA_ServerConfig_setDefaultWithSecurityPolicies(&config, 4840,
+                                                       &certificate, &privateKey,
+                                                       NULL, 0, NULL, 0, NULL, 0);
+    if(res != UA_STATUSCODE_GOOD)
+        goto cleanup;
+    config.certificateVerification.clear(&config.certificateVerification);
+    res = UA_CertificateVerification_CertFolders(&config.certificateVerification,
+                                                 trustlistFolder, issuerlistFolder,
+                                                 revocationlistFolder);
+    if(res != UA_STATUSCODE_GOOD)
+        goto cleanup;
+#endif /* __linux__ */
 
     if(!enableUnencr)
         disableUnencrypted(&config);
@@ -1191,6 +1272,23 @@ int main(int argc, char **argv) {
     if(disableBasic256Sha256)
         disableBasic256Sha256SecurityPolicy(&config);
 
+#else /* UA_ENABLE_ENCRYPTION */
+    UA_StatusCode res =
+        UA_ServerConfig_setMinimal(&config, 4840, &certificate);
+    if(res != UA_STATUSCODE_GOOD)
+        goto cleanup;
+#endif /* UA_ENABLE_ENCRYPTION */
+
+    if(!enableAnon)
+        disableAnonymous(&config);
+
+    /* Limit the number of SecureChannels and Sessions */
+    config.maxSecureChannels = 10;
+    config.maxSessions = 20;
+
+    /* Revolve the SecureChannel token every 300 seconds */
+    config.maxSecurityTokenLifetime = 300000;
+
     /* Set operation limits */
     config.maxNodesPerRead = MAX_OPERATION_LIMIT;
     config.maxNodesPerWrite = MAX_OPERATION_LIMIT;
@@ -1201,31 +1299,16 @@ int main(int argc, char **argv) {
     config.maxNodesPerNodeManagement = MAX_OPERATION_LIMIT;
     config.maxMonitoredItemsPerCall = MAX_OPERATION_LIMIT;
 
+    /* Set Subscription limits */
+#ifdef UA_ENABLE_SUBSCRIPTIONS
+    config.maxSubscriptions = 20;
+#endif
+
     /* If RequestTimestamp is '0', log the warning and proceed */
     config.verifyRequestTimestamp = UA_RULEHANDLING_WARN;
-
     if(enableTime)
         config.verifyRequestTimestamp = UA_RULEHANDLING_DEFAULT;
 
-#else
-    UA_ServerConfig_setMinimal(&config, 4840, &certificate);
-#endif
-
-    if(!enableAnon)
-        disableAnonymous(&config);
-
-    /* Clean up temp values */
-    UA_ByteString_clear(&certificate);
-#if defined(UA_ENABLE_ENCRYPTION) && !defined(__linux__)
-    UA_ByteString_clear(&privateKey);
-    for(size_t i = 0; i < trustListSize; i++)
-        UA_ByteString_clear(&trustList[i]);
-    for(size_t i = 0; i < issuerListSize; i++)
-        UA_ByteString_clear(&issuerList[i]);
-    for(size_t i = 0; i < revocationListSize; i++)
-        UA_ByteString_clear(&revocationList[i]);
-#endif
-
     /* Override with a custom access control policy */
     config.accessControl.getUserAccessLevel = getUserAccessLevel_disallowSpecific;
     UA_String_clear(&config.applicationDescription.applicationUri);
@@ -1234,15 +1317,35 @@ int main(int argc, char **argv) {
 
     config.shutdownDelay = 5000.0; /* 5s */
 
-    UA_Server *server = UA_Server_newWithConfig(&config);
-    if(server == NULL)
-        return EXIT_FAILURE;
+    server = UA_Server_newWithConfig(&config);
+    if(!server) {
+        res = UA_STATUSCODE_BADINTERNALERROR;
+        goto cleanup;
+    }
 
     setInformationModel(server);
 
     /* run server */
-    UA_StatusCode retval = UA_Server_run(server, &running);
+    res = UA_Server_run(server, &running);
 
-    UA_Server_delete(server);
-    return retval == UA_STATUSCODE_GOOD ? EXIT_SUCCESS : EXIT_FAILURE;
+ cleanup:
+    if(server)
+        UA_Server_delete(server);
+    else
+        UA_ServerConfig_clean(&config);
+
+    UA_ByteString_clear(&certificate);
+#if defined(UA_ENABLE_ENCRYPTION)
+    UA_ByteString_clear(&privateKey);
+#ifndef __linux__
+    for(size_t i = 0; i < trustListSize; i++)
+        UA_ByteString_clear(&trustList[i]);
+    for(size_t i = 0; i < issuerListSize; i++)
+        UA_ByteString_clear(&issuerList[i]);
+    for(size_t i = 0; i < revocationListSize; i++)
+        UA_ByteString_clear(&revocationList[i]);
+#endif
+#endif
+
+    return res == UA_STATUSCODE_GOOD ? EXIT_SUCCESS : EXIT_FAILURE;
 }
